#include "Graphics.h"
#include "CSDL.h"
#include <string.h>

CGfx gfx;

Color::Color(Uint8 _r, Uint8 _g, Uint8 _b, Uint8 _a)
{
    r = _r;
    g = _g;
    b = _b;
    a = _a;
}

Color::Color(Uint32 _argb)
{
    argb = _argb;
}

CGfx::CGfx()
{
    mPixels2D = NULL;
}

void CGfx::init(const char* title, int w, int h)
{
    sdl.Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
    mWidth = w;
    mHeight = h;
    mWindow = sdl.CreateWindow(title, SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, w, h, SDL_WINDOW_SHOWN);
    mRenderer = sdl.CreateRenderer(mWindow, -1, SDL_RENDERER_ACCELERATED);
}

void CGfx::init2D()
{
    mTexture2D = sdl.CreateTexture(mRenderer, SDL_PIXELFORMAT_ARGB8888, SDL_TEXTUREACCESS_STATIC, mWidth, mHeight);
    mPixels2D = new Color[mWidth * mHeight];
    memset(mPixels2D, 0, mWidth * mHeight * sizeof(Uint32));
}

void CGfx::quit()
{
    if (mPixels2D != NULL)
    {
        delete[] mPixels2D;
        sdl.DestroyTexture(mTexture2D);
    }
    sdl.DestroyRenderer(mRenderer);
    sdl.DestroyWindow(mWindow);
    sdl.Quit();
}

void CGfx::clearScreen(Color c)
{
    if (mPixels2D != NULL)
    {
        for (int i = 0; i < mWidth * mHeight; ++i)
            mPixels2D[i] = c;
    }
    else
    {
        sdl.SetRenderDrawColor(mRenderer, c.r, c.g, c.b, c.a);
        sdl.RenderClear(mRenderer);
    }
}

void CGfx::render()
{
    if (mPixels2D != NULL)
    {
        sdl.UpdateTexture(mTexture2D, NULL, mPixels2D, mWidth * sizeof(Uint32));
        sdl.RenderCopy(mRenderer, mTexture2D, NULL, NULL);
    }

    sdl.RenderPresent(mRenderer);
}

size_t CGfx::pixelPos2D(int x, int y)
{
    return y * mWidth + x;
}

void CGfx::setPixel(int x, int y, Color c)
{
    size_t  pos = pixelPos2D(x, y);
    mPixels2D[pos] = c;
}

Color CGfx::getPixel(int x, int y)
{
    size_t  pos = pixelPos2D(x, y);
    return mPixels2D[pos];
}
